/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/



/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "LPC17xx.h"
#include "type.h"
#include "uart.h"

/* TFC includes */
#include "printf.h"
#include "wifly.h"
#include "arpalab.h"
#include "cpdsurveyor.h"

/* Demo includes. */
#include "basic_io.h"

/*
/* buffer creado para imprimir el log por el cp2102 conectado en el uart3 */

char log[BUFSIZE];

/* El int Rssi es donde alojaremos en todo momento el Rssi para controlar la cobertura.
 * El int LimitRssi es donde configuraremos el límite de rssi aceptable */

int Rssi, LimitRssi;

// Las tareas que intervienen en la aplicación.

static void GetControlCPDSurveyor ( void *pvParameters );
static void MakeControlAndStatusCPDSurveyor ( void *pvParameters );

/* Declaramos una variable de tipo xQueueHandle. En esta variable se almacenará los datos recibidos
 * del interface de usuario y será accesible por las dos tareas */

xQueueHandle xQueue;

int main( void )
{
  /* ZONA INICIALIZACIÓN */

  /* Creamos la cola con un máximo de 4 elementos */

   xQueue = xQueueCreate( 4, sizeof( long ) );

  /* Inicializamos los dispositivos en los UART correspondientes con los baudrates correspondientes. */

  PrintInit(3,115200);
  WiflyInit(0,9600);
  MotorInit(1,115200);

  /* ZONA DE CONFIGURACIÓN */

  /* Insertaremos la password y bssid de la red donde nos vamos a conectar */

  WiflyConect("6K6FL5EN3QUXHNGM0CMK","WLAN_FDC4");

  /* Insertaremos el dominio y puerto donde se encuentra alojada la aplicación web (inteface usuario)*/

  CPDSurveyorConfig("www.onesistemas.com",80);

  /* Valor límite de rssi donde al sobrepasarlo el CPDSurveyor encontrará una zona con mejor cobertura */

  LimitRssi=-50;


	if( xQueue != NULL )
	{
		/* Creamos una tarea que será encargada de ir leyendo los datos para controlar el prototipo.
		  Periódicamente irá recogiendo datos desde www.onesistemas.com/CPDSurveyor/feed/
		  y los irá insertando en la cola creada anteriormente. */

		xTaskCreate( GetControlCPDSurveyor, "Control", 240, NULL, 1, NULL );

		/* Creamos una tarea que será la encargada de ir leyendo los datos de la cola y procesarlos.
		 * Lo primero que realizará será realizar el movimiento, posteriormente enviará de nuevo el dato
		 * procesado al servidor para informar del último movimiento realizado.
		 * En esta misma tarea se controlará el rssi y la cobertura del prototipo */

		xTaskCreate( MakeControlAndStatusCPDSurveyor, "Status", 240, NULL, 2, NULL );

		/* Start the scheduler so the created tasks start executing. */
		vTaskStartScheduler();
	}
	else
	{
		/* The queue could not be created. */
	}

  /* If all is well we will never reach here as the scheduler will now be
  running the tasks.  If we do reach here then it is likely that there was
  insufficient heap memory available for a resource to be created. */

	for( ;; );

	return 0;
}
/*-----------------------------------------------------------*/

static void GetControlCPDSurveyor ( void *pvParameters )
{
long lValueToSend;
portBASE_TYPE xStatus;

// Perform an action every 10 ticks.

portTickType xLastWakeTime;
const portTickType xFrequency = 50000;

	/* Two instances are created of this task so the value that is sent to the
	queue is passed in via the task parameter rather than be hard coded.  This way
	each instance can use a different value.  Cast the parameter to the required
	type. */

// Initialise the xLastWakeTime variable with the current time.
xLastWakeTime = xTaskGetTickCount();


	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{

		/* Delay implementado para probar con diferentes frecuencias de generación de esta tarea */
		// Wait for the next cycle.
		//vTaskDelayUntil( &xLastWakeTime, xFrequency );

		lValueToSend = CPDSurveyorGetDato();

		/* The first parameter is the queue to which data is being sent.  The 
		queue was created before the scheduler was started, so before this task
		started to execute.

		The second parameter is the address of the data to be sent.

		The third parameter is the Block time � the time the task should be kept
		in the Blocked state to wait for space to become available on the queue
		should the queue already be full.  In this case we don�t specify a block
		time because there should always be space in the queue. */

		xStatus = xQueueSendToBack( xQueue, &lValueToSend, 0 );

		if( xStatus != pdPASS )
		{

			/* Si no podemos escribir en la cola por que ya esta llena de datos
			 * nos aparecerá el siguiente mensaje por la consola de debug */

			sprintf(log,"********************************  LOG ControlCPDSurveyor: No se pudo enviar a la cola por que esta llena.\r\n");
			PrintText(3,log);
		}

		/* Allow the other sender task to execute. */
		taskYIELD();

	}
}
/*-----------------------------------------------------------*/

static void MakeControlAndStatusCPDSurveyor( void *pvParameters )
{
/* Declare the variable that will hold the values received from the queue. */
long lReceivedValue;

portBASE_TYPE xStatus;
const portTickType xTicksToWait = 12000 / portTICK_RATE_MS;


// Perform an action every 10 ticks.

portTickType xLastWakeTime;
const portTickType xFrequency = 50000;


// Initialise the xLastWakeTime variable with the current time.
xLastWakeTime = xTaskGetTickCount();

	/* This task is also defined within an infinite loop. */
	for( ;; )
	{
		/* Delay implementado para probar con diferentes frecuencias de generación de esta tarea */
		// Wait for the next cycle.
		// vTaskDelayUntil( &xLastWakeTime, xFrequency );

		/* As this task unblocks immediately that data is written to the queue this
		call should always find the queue empty. */
		if( uxQueueMessagesWaiting( xQueue ) != 0 )
		{
			/* Si intentamos enviar un dato pero la cola esta vaía nos aparecerá el
			 * siguiente mensaje por la consola de debug */

			sprintf(log,"********************************  LOG StatusCPDSurveyor: La cola debe estar vacia!\r\n");
			PrintText(3,log);

		}

		/* The first parameter is the queue from which data is to be received.  The
		queue is created before the scheduler is started, and therefore before this
		task runs for the first time.

		The second parameter is the buffer into which the received data will be
		placed.  In this case the buffer is simply the address of a variable that
		has the required size to hold the received data. 

		the last parameter is the block time � the maximum amount of time that the
		task should remain in the Blocked state to wait for data to be available should
		the queue already be empty. */
		xStatus = xQueueReceive( xQueue, &lReceivedValue, xTicksToWait );

		if( xStatus == pdPASS )
		{

			/* Enviamos el dato tipo long que hemos recibido de la cola al Arpalab.
			 * El envio es controlado por un semaforo para la compartición del dispositivo
			 * WiFly. */

				MotorControl(lReceivedValue);
				if (lReceivedValue==0){

						PrintText(3,"Motor parado\r");
						CPDSurveyorSendStatus(lReceivedValue);

					} else if (lReceivedValue==1){

						PrintText(3,"Motor movido hacia adelante\r");
						CPDSurveyorSendStatus(lReceivedValue);

					} else if (lReceivedValue==2){

						PrintText(3,"Motor movido hacia la derecha\r");
						CPDSurveyorSendStatus(lReceivedValue);

					} else if (lReceivedValue==3){

						PrintText(3,"Motor movido hacia la izquierda\r");
						CPDSurveyorSendStatus(lReceivedValue);

					} else if (lReceivedValue == 4){

						PrintText(3,"Motor movido atras\r");
						CPDSurveyorSendStatus(lReceivedValue);

					}

				/* Trás haber realizado el último movimiento comprobamos si tenemos una cobertura aceptable */

				Rssi=WiflyRssi(); //Obtenemos el RSSI y comprobamos
				if (Rssi<=LimitRssi){

				PrintText(3,"Mala calidad\r");
				CPDSurveyorSendRssi(Rssi); //enviamos el rssi de mala calidad y la aplicació nos informará que entra en modo automático

				//realizamos el control de cobertura trás haber realizado un movimiento para nunca perder la cobertura

				MotorCobertura(Rssi, LimitRssi);

				} else {

				PrintText(3,"Buena calidad\r");
				CPDSurveyorSendRssi(Rssi); //enviamos el rssi de buena calidad e informamos en la interface web que es correcto.

				}


			/* Imprimimos por el Uart 3 el dato enviado */

			sprintf(log,"********************************  LOG StatusCPDSurveyor: Dato %d enviado!\r\n",lReceivedValue);
			PrintText(3,log);

		}
		else
		{
			/* Si no recibimos ningún dato de la cola tras esperar los TickstoWait (12000 ms) nos
			 * aparecerá continuamente el siguiente mensaje informativo. */

			sprintf(log,"********************************  LOG StatusCPDSurveyor: No se pudo recibir datos de la cola.\r\n");
			PrintText(3,log);
		}

	}
}


/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


