/*
 * arpalab.c
 *
 * Driver/Librería para configurar la aplicación web "Arpalab" con el dispositivo WiFly
 *
 *  Created on: 01/12/2012
 *      Author: jaivar
 */

/* Librerias implicadas en este driver */

#include "LPC17xx.h"
#include "type.h"
#include "uart.h"
#include "printf.h"
#include "wifly.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



char buffRx[BUFSIZE]; // buffer para recibir respuesta del Driver a través del UART
char buffTx[BUFSIZE]; // buffer para enviar comando al driver a través del UART
char buffLog[BUFSIZE]; // buffer para imprimir por el log la respuesta del UART


uint32_t sizeAsk; // int para alojar la cantidad de carácteres recibidos por respuesta en cada comando enviado al wifly.

/*
 *
 * La fución "ArpalabConfig" tiene como objetivo configurar en nuestro dispositivo
 * Wifly el servidor y el puerto donde escucha la aplicación web Arpalab.
 * También configurará el modo en que enviaremos y recibiremos los datos.
 *
 * Parámetros de entrada:
 *
 * Dnsname = Nombre de dominio donde se encuentra alojada la aplicación web.
 * Port = Puerto donde escucha la aplicación.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */


uint32_t ArpalabConfig(char *Dnsname, int Port){

		PrintText(3,"\r\n********** Comandos GET via http **********\r\n");

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip proto 18");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set dns name %s",Dnsname);
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip host 0");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip remote %d", Port);
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set option format 1");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		return 0;
}

/*
 *
 * La fución "ArpalabGetDato" tiene como objetivo recibir el dato que nos interesa de la aplicación Arpalab.
 * Para ello primero deberemoss configurar la ruta en el wifly donde se encuentra la página y donde deseamos
 * extraer el dato.
 * Trás descargar la página, se realiza un "parser" personalizado para obtener el dato que nos interesa,
 * que en este caso es un número aleatorio tipo long.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * Dato = El dato tipo long que deseamos.
 *
 */

long ArpalabGetDato(){

		// Creamos las variables y buffers que necesitamos para crear el Parser
		char PrefixCorte[4]="Ca";
		char BuffParser[BUFSIZE];
		int Dato;

		// Introducimos la ruta en el Wifly
		sprintf(buffTx,"set com remote GET$/rest/feed");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos un open y esperamos 10 seg. a que se pueda leer correctamente la página con el dato.
		sprintf(buffTx,"open");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,10000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos el parser y extraemos el dato descargado
		sprintf(BuffParser,"%s", strstr(buffRx,PrefixCorte)+26);
		strtok(BuffParser,"*CLOS*");
		Dato = atoi(BuffParser);
		sprintf(buffLog,"El dato recibido es: %d \n",Dato);
		PrintText(3,buffLog);

		//Entramos en modo comando para salír del open
		PrintText(3,"\r\n********** Entrando en modo comando.... **********\r\n");
		WiflyConsola(0);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Devolvemos el dato obtenido.
		return Dato;
}

/*
 *
 * La fución "ArpalabSendDato" tiene como objetivo enviar el dato que préviamente hemos
 * obtenido con la función ArpalabGetDato. Para ello deberemos configurar la ruta en el wifly
 * donde la aplicación Arpalab leerá los datos que enviemos.
 *
 * Parámetros de entrada:
 *
 * Dato = Nombre de dominio donde se encuentra alojada la aplicación web.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t ArpalabSendDato( uint32_t Dato){

		// Introducimos la ruta en el Wifly donde se enviará el dato además de otras variables como la ip y el id
		sprintf(buffTx,"set com remote GET$/rest/stats?nid=160779&ip=213.22.111.5&app=jaivar&data=%d",Dato);
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos un open y esperamos 10 seg. para que se pueda leer correctamente la página de conformacion.
		sprintf(buffTx,"open");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,10000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Entramos en modo comando para salír del open
		PrintText(3,"\r\n********** Entrando en modo comando.... **********\r\n");
		WiflyConsola(0);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		return 0;
}
