/*
 * cpdsurveyor.c
 *
 * Driver/Librería para configurar la aplicación web "CPDSurveyor" con el dispositivo WiFly
 *
 *  Created on: 01/12/2012
 *      Author: jaivar
 */

/* Librerias implicadas en este driver */

#include "LPC17xx.h"
#include "type.h"
#include "uart.h"
#include "printf.h"
#include "wifly.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


char buffRx[BUFSIZE]; // buffer para recibir respuesta del Driver a través del UART
char buffTx[BUFSIZE]; // buffer para enviar comando al driver a través del UART
char buffLog[BUFSIZE]; // buffer para imprimir por el log la respuesta del UART

uint32_t sizeAsk; // int para alojar la cantidad de carácteres recibidos por respuesta en cada comando enviado al wifly.

/*
 *
 * La fución "CPDSurveyorConfig" tiene como objetivo configurar en nuestro dispositivo
 * Wifly el servidor y el puerto donde escucha la aplicación web CPDSurveyor.
 * También configurará el modo en que enviaremos y recibiremos los datos.
 *
 * Parámetros de entrada:
 *
 * Dnsname = Nombre de dominio donde se encuentra alojada la aplicación web.
 * Port = Puerto donde escucha la aplicación.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */
uint32_t CPDSurveyorConfig(char *Dnsname, int Port){

		PrintText(3,"\r\n********** Comandos GET via http **********\r\n");

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip proto 18");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set dns name %s",Dnsname);
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip host 0");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set ip remote %d", Port);
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		//Preparamos comando y enviamos al Wifly
		sprintf(buffTx,"set option format 1");
		WiflyComand(0,buffTx);

		//Imprimimos la respuesta para tener un log
		WiflyLogResponse(1000,3);

		return 0;
}

/*
 *
 * La fución "CPDSurveyorGetDato" tiene como objetivo recibir el dato que nos interesa de la aplicación CPDSurveyor.
 * Para ello primero deberemos configurar la ruta en el wifly donde se encuentra la página y donde deseamos
 * extraer el dato.
 * Trás descargar la página, se realiza un "parser" personalizado para obtener el dato que nos interesa,
 * que en este caso es un número aleatorio tipo long.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 *  Dato = El dato tipo long que deseamos.
 *
 */

long CPDSurveyorGetDato(){

		// Creamos las variables y buffers que necesitamos para crear el Parser
		char PrefixCorte[4]="1>";
		char BuffParser[BUFSIZE];
		int Dato;

		// Introducimos la ruta en el Wifly
		sprintf(buffTx,"set com remote GET$/CPDSurveyor/feed/index.php");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos un open y esperamos 10 seg. a que se pueda leer correctamente la página con el dato.
		sprintf(buffTx,"open");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,10000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos el parser y extraemos el dato descargado
		sprintf(BuffParser,"%s", strstr(buffRx,PrefixCorte)+2);
		Dato = atoi(BuffParser);
		sprintf(buffLog,"El dato recibido es: %d \n",Dato);
		PrintText(3,buffLog);

		// Entramos en modo comando para salír del open
		PrintText(3,"\r\n********** Entrando en modo comando.... **********\r\n");
		WiflyConsola(0);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Devolvemos el dato obtenido.
		return Dato;
}

/*
 *
 * La fución "CPDSurveyorSendRssi" tiene como objetivo enviar el rssi que préviamente habremos
 * obtenido con la función WiflyRssi(). Para ello deberemos configurar la ruta en el wifly
 * donde la aplicación CPDSurveyor leerá el dato que le enviemos.
 *
 * Parámetros de entrada:
 *
 * Dato = Valor int con el Rssi a enviar a la interface de usuario.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t CPDSurveyorSendRssi( uint32_t Dato){

		// Introducimos la ruta en el Wifly donde se enviará el dato con el valor del Rssi
		sprintf(buffTx,"set com remote GET$/CPDSurveyor/registra.php?rssi=%d",Dato);
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos un open y esperamos 10 seg. para que se pueda leer correctamente la página de conformacion.
		sprintf(buffTx,"open");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,10000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Entramos en modo comando para salír del open
		PrintText(3,"\r\n********** Entrando en modo comando.... **********\r\n");
		WiflyConsola(0);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		return 0;
}

/*
 *
 * La fución "CPDSurveyorSendStatus" tiene como objetivo enviar el dato que préviamente hemos
 * obtenido con la función ArpalabGetDato. Este dato nos indica el movimiento que ha realizado
 * el prototipo pudiendo así enviar la confirmación del movimiento a la interface de usuario.
 *
 * Parámetros de entrada:
 *
 * Dato = número que indica la dirección (0 = Parar, 1 = Adelante, 2 = Derecha, 3 = Izquierda, 4 = Atrás)
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t CPDSurveyorSendStatus( uint32_t Control){

		// Introducimos la ruta en el Wifly donde se enviará el dato
		sprintf(buffTx,"set com remote GET$/CPDSurveyor/status.php?statusmotor=%d",Control);
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		// Realizamos un open y esperamos 10 seg. para que se pueda leer correctamente la página de conformación.
		sprintf(buffTx,"open");
		WiflyComand(0,buffTx);
		sizeAsk=WiflyUART0scanf(buffRx,10000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		//Entramos en modo comando para salír del open
		PrintText(3,"\r\n********** Entrando en modo comando.... **********\r\n");
		WiflyConsola(0);
		sizeAsk=WiflyUART0scanf(buffRx,1000);
		sprintf(buffLog,"hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
		PrintText(3,buffLog);

		return 0;
}
