/*
 * printf.c
 *
 *  Created on: 12/10/2012
 *      Author: jaivar
 */


/* Librerias implicadas en este driver */

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "type.h"
#include "uart.h"
#include <stdio.h>
#include <string.h>

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];

/* Declaramos una variable de tipo xSemaphoreHandle. Esta se usará para el mutex
 * del semáfomo que nos asegurará una protección de exclusión mutua para imprimir
 * por el Uart seleccionado.
 */

xSemaphoreHandle xMutexPrint;

/*
 *
 * La fución "PrintInit" tiene como objetivo inicializar el cp2102 en el sistema,
 * por lo que será de uso obligatorio para el correcto funcionamiento y deberá
 * ser llamada antes de crear las tasks.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 * BaudRate = baudrate para poder negociar con el driver.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t PrintInit (int UartPort, int BaudRate){

	 // Pasamos como parametros el uart id & baud rate setting del UART
	 UARTInit(UartPort, BaudRate);

	 // Aprovechamos esta función para crear el semáforo antes de utilizarlo.
	 xMutexPrint = xSemaphoreCreateMutex();

	 return 0;
}

/*
 *
 * La fución "PrintText" tiene como objetivo enviar un buffer de texto por el
 * puerto uart donde tenemos conectado el cp2102.
 * Tiene un semáforo para la protección anteriormente comentada.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el cp2102.
 * TextOut = buffer de texto que deseamos enviar.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t PrintText (int UartPort,char *TextOut)
{
	xSemaphoreTake( xMutexPrint, portMAX_DELAY );
			{
    UART3Count=0;
    UARTSend(UartPort, (uint8_t *)TextOut , strlen(TextOut) );
			}
	xSemaphoreGive( xMutexPrint );

   return 0;

}

/*
 *
 * La fución "PrintTextAndNum" tiene como objetivo enviar un buffer de texto y
 * un número por el puerto uart donde tenemos conectado el cp2102.
 * Tiene un semáforo para la protección anteriormente comentada.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el cp2102.
 * TextIn = buffer de texto que deseamos enviar.
 * NumIn = número que deseamos enviar conjuntamente con el texto.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t PrintTextAndNum (int UartPort, char *TextIn, unsigned long NumIn)
{

   // buffer que utilizaremos para juntar el texto con el número y poder enviarlo.
   char buffer[BUFSIZE];

   printf("%s %u\n",TextIn, NumIn);
   sprintf(buffer, "\r%s %u\r\n", TextIn, NumIn);

   xSemaphoreTake( xMutexPrint, portMAX_DELAY );
   			{
   UART3Count=0;
   UARTSend(UartPort, (uint8_t *)buffer , strlen(buffer) );
   			}
   xSemaphoreGive( xMutexPrint );

   return 0;
}
